import std/options
import bearssl

import ../jwt/jwa
import ../jwt/jwk

import ./crypto/hash
import ./crypto/hmac

proc validateAlgMatchesKey(alg: JwtAlgorithm, key: JWK) =
  if key.alg.isSome and key.alg.get != $alg:
    raise newException(ValueError,
      "requested signature algorithm (" & $alg & ") does not match the " &
      "algorithm intended for use with this key (" & key.alg.get & ")")

proc computeSignature*(
    payload: string,
    alg: JwtAlgorithm,
    key: JWK
  ): string =

  validateAlgMatchesKey(alg, key)

  case alg:
    of HS256, HS384, HS512: return hmac(alg, key, payload)
    else:
      raise newException(Exception,
        "unsupported signature algorithm: " & $alg)

proc verifySignature*(
    payload: string,
    signature: string,
    alg: JwtAlgorithm,
    key: JWK
  ): bool =

  validateAlgMatchesKey(alg, key)

  case alg:
    of HS256, HS384, HS512:
      let hash = hmac(alg, key, payload)
      return hash == signature
    else:
      raise newException(Exception,
        "unsupported signature algorithm: " & $alg)
