import std/json, std/unittest

import jwt/claims, jwt/joseheader, jwt/jwa, jwt/jwk, jwt/jws, jwt/jwssig

import private/encoding

import ../testdata

suite "jwt/jws":

  test "HS256 - sign":

    check rfc7515A1ExampleJwt == $sign(
      header = initJoseHeader(rfc7515A1ExampleHeaderB64),
      payload = byteArrToString(rfc7515A1ExampleClaimsBytes),
      key = initJwk(parseJson(rfc7515A1ExampleJwkStr)))

    check sampleJwt == $sign(
      header = initJoseHeader(parseJson(sampleJwtHeaderDecoded)),
      payload = $initJwtClaims(parseJson(sampleJwtClaimsDecoded)),
      key = initJwk(parseJson(sampleJwtKey)))

  test "HS256 - verify":

    validate(
      jws = initJWS(rfc7515A1ExampleJwt),
      alg = HS256,
      key = initJwk(parseJson(rfc7515A1ExampleJwkStr)))

    validate(
      jws = initJWS(sampleJwt),
      alg = HS256,
      key = initJwk(parseJson(sampleJwtKey)))

  test "HS256 - round trip":

    let payload = "This is a message I want to protect from tampering."
    let jwk = initJwk(%*{
      "kty": "oct",
      "alg": "HS256",
      "k": b64UrlEncode("This is my secret key")
    })

    let jws = sign(
      header = initJoseHeader(%*{ "alg": "HS256" }),
      payload = payload,
      key = jwk)

    jws.validate(HS256, jwk)
